/**
 * <p>Copyright: Copyright (c) 2005, Knowledge Science Group, University of Calgary.
 * Permission to use, copy, modify, distribute and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting documentation.  The
 * Knowledge Science Group makes no representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express or
 * implied warranty.</p>
 */

package kartaCASA;

import kartaCASA.interfaces.KartaAgentInterface;
import casa.Agent;
import casa.ML;
import casa.exceptions.IPSocketException;
import casa.util.TypeHierarchy;

/**
 * KartaAgent is a superclass for the 3 types (model-view-supervisor) of Karta agents.
 * 
 * Created on June 27, 2005
 * @author <a href="mailto:sedachv@cpsc.ucalgary.ca">Vladimir Sedach</a>
 * @version 0.04
 */
public class KartaAgent extends Agent implements KartaAgentInterface {
	
	private boolean debugp = true;
	
	public KartaAgent(Qualifiers quals) throws IPSocketException {
		super(quals);
	}
	
	protected TypeHierarchy makeDefaultActs (TypeHierarchy a) {
		in ("KartaAgent.makeDefaultActs()");
		a = super.makeDefaultActs(a);
		try {
			a.add(ML.ACT_CONSIDER, ML.ACT_ACT);
			a.add(ACT_CONSIDER_COMMAND, ML.ACT_CONSIDER);
			a.add(ACT_GET_STATE, ML.ACT_GET);
			a.add(ACT_SET, ML.ACT_ACT);
			a.add(ACT_SET_STATE, ACT_SET);
			a.add(ACT_SET_MAPPING, ACT_SET);
			a.add(ACT_JOIN_VIEW, new String[] {ML.ACT_REGISTER, ML.ACT_CHANGE_MEMBERSHIP});
			a.add(ACT_WITHDRAW_VIEW, new String[] {ML.ACT_UNREGISTER, ML.ACT_CHANGE_MEMBERSHIP});
			a.add(ACT_JOIN_MODEL, new String[] {ML.ACT_REGISTER, ML.ACT_CHANGE_MEMBERSHIP});
			a.add(ACT_WITHDRAW_MODEL, new String[] {ML.ACT_UNREGISTER, ML.ACT_CHANGE_MEMBERSHIP});
		}
		catch (Throwable e) {
			System.out.println("Warning: error initializing acts for Karta agent.");
		}
		return a;
	}
}
