/**
 * <p>Copyright: Copyright (c) 2005, Knowledge Science Group, University of Calgary.
 * Permission to use, copy, modify, distribute and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting documentation.  The
 * Knowledge Science Group makes no representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express or
 * implied warranty.</p>
 */
package kartaCASA.MASExplorer;

import java.util.Hashtable;

import casa.URLDescriptor;

import karta.util.ObjectID;
import karta.Pickler.StringTypeWrapper;

import kartaCASA.ModelViewCommandTranslator;

/**
 * Command translator between Karta Views and the MASExplorer model.
 * 
 * Created on July 6, 2005
 * @author <a href="mailto:sedachv@cpsc.ucalgary.ca">Vladimir Sedach</a>
 * @version 0.02
 */
public class MASCommandTranslator extends ModelViewCommandTranslator {

	
	Hashtable<ObjectID, String> IDtoAgentMap;
	Hashtable<String, ObjectID> AgenttoIDMap;
	
	public MASCommandTranslator() {
		IDtoAgentMap = new Hashtable<ObjectID, String>();
		AgenttoIDMap = new Hashtable<String, ObjectID>();
	}
	
	public ObjectID getID(Object agent) {
		if (agent instanceof URLDescriptor)  {
			return getID((URLDescriptor)agent);
		}
		else if (agent instanceof String) {
			return getID((String)agent);
		}
		else if (agent instanceof StringTypeWrapper) {
			return getID(((StringTypeWrapper)agent).getString());
		}
		else {
			return null;
		}
	}
	
    public ObjectID getID(URLDescriptor agent) {
    	return getID(agent.toString());
    }
    
    public ObjectID getID(String agent) {
    	return AgenttoIDMap.get(agent);
    }
    
    public Object getModelEntity(ObjectID ID) {
    	return getModelEntity(ID.toString());
    }
    
    public Object getModelEntity(String id) {
    	URLDescriptor url = null;
    	try {
    		url = new URLDescriptor(IDtoAgentMap.get(id));
    	}
    	catch (Throwable e) {}
    	return url;
    }
    
    public void MapEntityToID(Object agent, ObjectID ID) {
    	AgenttoIDMap.put(agent.toString(), ID);
    }
    
    public void MapIDToEntity(ObjectID ID, Object agent) {
    	IDtoAgentMap.put(ID, agent.toString());
    }
}
