/**
 * <p>Copyright: Copyright (c) 2005, Knowledge Science Group, University of Calgary.
 * Permission to use, copy, modify, distribute and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting documentation.  The
 * Knowledge Science Group makes no representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express or
 * implied warranty.</p>
 */
package kartaCASA.MASExplorer;

import java.util.Hashtable;

import casa.URLDescriptor;
import casa.exceptions.IPSocketException;

import karta.Pickler.StringTypeWrapper;
import karta.command.Command;
import karta.util.IDMapper;
import karta.util.ObjectID;

import kartaCASA.KartaViewAgent;

/**
 * A Karta view agent with MASExplorer-specific functionality.
 * 
 * Created on July 8, 2005
 * @author <a href="mailto:sedachv@cpsc.ucalgary.ca">Vladimir Sedach</a>
 * @version 0.04
 */
public class MASViewAgent extends KartaViewAgent {
	
	boolean debugp = false;
	
	/** A table of all the agents that are already present in the views, used so that if one of the
	 * agents joins another URLDescriptor, a duplicate node will not be created. */
	protected Hashtable<URLDescriptor, Object> knownAgents;
	
	public MASViewAgent(Qualifiers quals) throws IPSocketException {
		super(quals);
		commandFilter = new MASModelFilter();
		commandTranslator = new MASCommandTranslator();
		knownAgents = new Hashtable<URLDescriptor, Object>();
	}
	
	protected void mapModelCommandID(final Command command) {
		Command.mapCommandIDs(command, 
				new IDMapper() { 
			public Object mapID(Object sID) {
				// the ID field will always have a printed URLDescriptor from the MASModelAgent
				// what this method does is to check whether we already know about the agent with
				// that URLDescriptor, and if so, does nothing, but if it is a new agent, then we
				// request that a new ID be assigned to it
				if (command.getType() == Command.ADD_NODE) {
					try {
						String modelID = ((StringTypeWrapper)command.getCommandArgument("modelID")).getString();
						URLDescriptor agent = new URLDescriptor(((StringTypeWrapper)sID).getString()); // this also ensures that sID really is a URLDescriptor
						if (debugp)
							System.out.println("Now mapping agent " + agent + " with modelID " + modelID);
						if (knownAgents.get(agent) == null) {
							if (agent.equals(modelID)) { // if we're mapping the actual ID instead of some supposed ID
								knownAgents.put(agent, this); // now we know about this agent
								return ObjectID.newNode.toString(); // next pass will actually assign a new ID
							}
						}
					}
					catch (Throwable e) {
						if (debugp)
							System.out.println("While MASViewAgent was mapping ID " + sID + " this happened " + e);
					}
				}
				return sID;
			}
		});
		super.mapModelCommandID(command);
	}
}
