package kartaCASA.OntologyEditor;

import kartaCASA.KartaSupervisorAgent;
import kartaCASA.ui.OntologyEditorKartaUI;

import karta.KartaInstance;

import casa.Agent.Qualifiers;
import casa.TransientAgent;
import casa.exceptions.IPSocketException;
import casa.ui.AgentUI;
import casa.util.DEBUG;

public class OntologyEditor {
	
	boolean debugp = false;
	
	public OntologyEditor(TransientAgent owner) throws IPSocketException {
		
		if (debugp)
			System.out.println("OntologyEditor now starting intialization.");
		
		OntologyEditorView viewAgent = null;
		
		// Initialize the view agent
		try {
			viewAgent = new OntologyEditorView(new casa.Agent.Qualifiers(0, "OntView", 9000), owner);
		}
		catch (Exception e) {
			System.out.println("OntologyEditor couldn't initialize viewAgent because " + e);
		}
			
		if (debugp)
			System.out.println("OntologyEditor has initialized the view agent " + viewAgent);
		
		Thread t = (Thread)viewAgent;
		t.start();
		while (!viewAgent.isInitialized()) {
			try {
				owner.sleep(500);
			}
			catch (InterruptedException e) {}
		}
		
		// initialize the supervisor agent
		KartaInstance kinstance = new KartaInstance() {
			protected void initGUI() {
				setUI(new OntologyEditorKartaUI(this));
			}
		};
		
		if (debugp)
			System.out.println("OntologyEditor finished intializing Karta instance " + kinstance);
		
		String[] supervisorArgs = {"-w", "-mKQML", "-nOntologyEditor", "+Ssc"};
		Qualifiers supervisorQualifiers = new casa.Agent.Qualifiers(supervisorArgs);
		OntologyEditorSupervisor supervisor = new OntologyEditorSupervisor(supervisorQualifiers, kinstance);
//		KartaSupervisorAgent supervisor = new KartaSupervisorAgent(supervisorQualifiers, kinstance);
		
		if (debugp)
			System.out.println("OntologyEditor made a supervisor agent " + supervisor);
		
		kinstance.setSupervisor(supervisor);
		
		Thread t1 = (Thread)supervisor;
		t1.start();
		while (!supervisor.isInitialized()) {
			try {
				owner.sleep(500);
			}
			catch (InterruptedException e) {}
		}
		
		// subscribe supervisor to view
		supervisor.subscribeToView(viewAgent.getURL());
		
		// make supervisor GUI
		AgentUI agentUI = null;
		try {
			agentUI = supervisor.makeDefaultInterface(supervisorArgs, "sedachv", true);
		}
		catch (Exception ex3) {
			DEBUG.DISPLAY_ERROR("Failed to start up default interface for supervisor", ex3);
		}
		
		if (agentUI!=null) {
			supervisor.setUI(agentUI);
			if (debugp)
				System.out.println("OntologyEditor made a UI for the supervisor agent " + agentUI);
		}
	}
}
