/**
 * <p>Copyright: Copyright (c) 2005, Knowledge Science Group, University of Calgary.
 * Permission to use, copy, modify, distribute and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting documentation.  The
 * Knowledge Science Group makes no representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express or
 * implied warranty.</p>
 */
package kartaCASA;

import java.util.Vector;

import kartaCASA.interfaces.KartaAgentInterface;

import casa.URLDescriptor;

import karta.command.Command;
import karta.Pickler.Pickler;

/**
 * The PersistentViewProxy stores the state of "sleeping" view agents.
 * 
 * Created on July 8, 2005
 * @author <a href="mailto:sedachv@cpsc.ucalgary.ca">Vladimir Sedach</a>
 * @version 0.04
 */
public class PersistentViewProxy implements KartaAgentInterface {
	/** The address of the view represented by this proxy. */
	private URLDescriptor url;
	/** The pickle of the last known complete concept map state for the ViewAgent. */
	private String latestState;
	/** List of model changes since latest_state. */
	private Vector<String> modelChanges;
	/** Sequence number of the last command processed by the view before sleeping?????. */
	private long sequenceNumber;
	/** Flag that signifies whether the view represented by this proxy is sleeping (offline) or not. */
	private boolean sleeping;
//	 /** the largest ID so far assigned - will be used later when sleeping is implemented. */
	//private String bigID; 
	private Pickler pickler;
	
	protected ModelViewCommandTranslator translator;
	
	public PersistentViewProxy(URLDescriptor url, String state, long num, Pickler pick) {
		this.url = url;
		this.latestState = state;
		this.modelChanges = new Vector<String>();
		this.sequenceNumber = num;
		this.sleeping = false;
		this.pickler = pick;
	}
	
	public String getLatestState() {
		return latestState;
	}
	
	public void setLatestState(String latestState) {
		this.latestState = latestState;
	}
	
	public boolean equals(Object u) {
		if (u instanceof URLDescriptor)
			return url.equals(u);
		else
			return super.equals(u);
	}
	
	Vector<String> getModelChanges() {
		return modelChanges;
	}
	
	/** This method is called to add a command that has been broadcast by the model
	 * to the list of such commands kept by the proxy (both when it is sleeping and
	 * when it is not). */
	void addModelChange(String com) {
		modelChanges.add(com);
	}
	
	public void putToSleep() {
		sleeping = true;
	}
	
	public void wake() {
		sleeping = false;
	}
	
	public boolean isSleeping() {
		return sleeping;
	}
	
	/** Prepares a command to be sent out to the view. 
	 * Should be called when the view is sleeping without actually sending the command. */
	public String prepCommand(Command com) {
		if (sleeping)
			translator.translateCommand(com);
		String command = pickler.pickleCommand(com);
		addModelChange(command);
		return command;
	}
	
	public ModelViewCommandTranslator getTranslator() {
		return translator;
	}
	
	public void setTranslator(ModelViewCommandTranslator translator) {
		this.translator = translator;
	}
	
	public URLDescriptor getURL() {
		return url;
	}
	
	public String toString() {
		return url.toString();
	}
	
	/** Does a search to find the given object (either a view proxy or a URL of a view) 
	 * in the given Vector of PersistentViewProxies. Returns null if not found. */
	public static PersistentViewProxy findProxy (Vector<PersistentViewProxy> proxies, Object obj) {
		for (int i = 0; i < proxies.size(); i++) {
			if (proxies.get(i).equals(obj))
				return proxies.get(i);
		}
		return null;
	}
}
