/**
 * <p>Copyright: Copyright (c) 2005, Knowledge Science Group, University of Calgary.
 * Permission to use, copy, modify, distribute and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting documentation.  The
 * Knowledge Science Group makes no representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express or
 * implied warranty.</p>
 */
package kartaCASA.ui;

import karta.KartaInstance;
import karta.dataStructures.*;
import karta.util.ObjectID;
import karta.UI.*;
import karta.command.*;

import java.awt.MenuItem;
import java.awt.event.*;

/**
 * A customized Karta UI specifically for use by the ontology editor.
 * 
 * Created on August 22, 2005
 * @author <a href="mailto:sedachv@cpsc.ucalgary.ca">Vladimir Sedach</a>
 * @version 0.03
 */
public class OntologyEditorKartaUI extends UserInterface {
	
	public OntologyEditorKartaUI(KartaInstance k) {
		super(k);
	}
	
	protected void saveOntology() {
		Command save = karta.getCommandMaker().saveChanges();
		karta.getSupervisor().considerCommand(save);
	}
	
	protected void restoreMap() {
		Command restore = karta.getCommandMaker().restoreMap();
		karta.getSupervisor().considerCommand(restore);
	}
	
	protected void addFileMenuItems(KartaPopupMenu menu) {
		MenuItem saveOntology = new MenuItem("Save ontology");
		MenuItem restoreOntology = new MenuItem("Restore ontology");
		
		class SaveOntology implements ActionListener {
			public void actionPerformed(ActionEvent e) {
				// send command to save ontology
				saveOntology();
			}
		}
		
		class RestoreOntology implements ActionListener {
			public void actionPerformed(ActionEvent e) {
				// send command to re-read ontology, discarding any changes
				restoreMap();
			}
		}
		
		saveOntology.addActionListener(new SaveOntology());
		restoreOntology.addActionListener(new RestoreOntology());
		
		menu.add(saveOntology);
		menu.add(restoreOntology);
		menu.addSeparator();
	}
	
	protected void deleteNode() {
		// get currently selected node, make command to delete it
		LinkedListObject selected = karta.getConceptMap().getSelectedObjects().get(0);
		if ((selected != null) && (selected instanceof NodeObject))
			DeleteGraphicsObjects();
	}
	
	protected void doCollapseNode (NodeObject node) {
		if (node != null) {
			karta.getSupervisor().considerCommand(karta.getCommandMaker().collapseNode(node));
			karta.getSupervisor().considerCommand(karta.getCommandMaker().setSelectedObject(ObjectID.invalid));
			karta.getConceptMap().setSelectedObject(null);
			panel.repaint();
		}
	}
	
	protected void collapseNode() {
		// get currently selected node, make command to collapse it
		LinkedListObject selected = karta.getConceptMap().getSelectedObjects().get(0);
		if ((selected != null) && (selected instanceof NodeObject))
			doCollapseNode((NodeObject) selected);
	}
	
	protected void addCreateEditMenuItems(Class type, KartaPopupMenu menu) {
		if (type == NodeObject.class) { // a node object is selected
			MenuItem createChild = new MenuItem("Make child node");
			MenuItem deleteNode = new MenuItem("Delete node");
			MenuItem collapseNode = new MenuItem("Collapse node");
			
			class CreateChild implements ActionListener {
				KartaPopupMenu mouse;

				CreateChild(KartaPopupMenu mouse) {
					super();
					this.mouse = mouse;
				}

				public void actionPerformed(ActionEvent e) {
					CreateNode(mouse.getposx(), mouse.getposy());
				}
			}
			
			class DeleteNode implements ActionListener {
				public void actionPerformed (ActionEvent e) {
					// send command to delete node 
					deleteNode();
				}
			}
			
			class CollapseNode implements ActionListener {
				public void actionPerformed (ActionEvent e) {
					// send command to collapse node
					collapseNode();
				}
			}
			
			createChild.addActionListener(new CreateChild(menu));
			deleteNode.addActionListener(new DeleteNode());
			collapseNode.addActionListener(new CollapseNode());
			
			menu.add(createChild);
			menu.add(deleteNode);
			menu.add(collapseNode);
			menu.addSeparator();
		}
	}
}
