/**
 * <p>Copyright: Copyright (c) 2005, Knowledge Science Group, University of Calgary.
 * Permission to use, copy, modify, distribute and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting documentation.  The
 * Knowledge Science Group makes no representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express or
 * implied warranty.</p>
 */

package kartaCASA.ui;

import casa.ui.AgentInternalFrame;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.MouseInputAdapter;

import java.util.Vector;

import casa.URLDescriptor;

import kartaCASA.KartaViewAgent;

/**
 * The user interface for a Karta view.
 * 
 * Created on June 15, 2005
 * @author <a href="mailto:sedachv@cpsc.ucalgary.ca">Vladimir Sedach</a>
 * @version 0.03
 */
public class ViewAgentWindow extends AgentInternalFrame {

    KartaViewAgent view;

    // View window needs to have a list of subscribing supervisors, a menu for subscribing to different model agents,
    // a list of commands under consideration, differentiating where commands go (model, rejected, or view shortcut)

    public ViewAgentWindow(KartaViewAgent v, Container frame) {
        super(v, "Karta View Agent: ", frame);

        this.view = v;

        pack();
    }
    
    /** Makes a CD panel like in TransientAgentInternalFrame, except here there is a right-click context-sensitive
     * menu for selecting CD participants to subscribe to as models.
     */
    protected JPanel makeCDPanel (int i) {
        JPanel panelCDparticipants = new JPanel ();
        JPanel panelCDlist = new JPanel ();
        JSplitPane panelCDtop = new JSplitPane (JSplitPane.HORIZONTAL_SPLIT,
            panelCDlist, panelCDparticipants);
        panelCDtop.setResizeWeight (0.5);
        panelCDtop.setDividerSize (4);

        panelCDlist.setLayout (new BorderLayout ());
        JLabel labelCDlist = new JLabel ();
        panelCDlist.add (labelCDlist, BorderLayout.NORTH);
        JScrollPane scrollCDlist = new JScrollPane ();
        panelCDlist.add (scrollCDlist, BorderLayout.CENTER);

        panelCDparticipants.setLayout (new BorderLayout ());
        JLabel labelCDparticipants = new JLabel ();
        panelCDparticipants.add (labelCDparticipants, BorderLayout.NORTH);
        JScrollPane scrollCDparticipants = new JScrollPane ();
        panelCDparticipants.add (scrollCDparticipants, BorderLayout.CENTER);

        while(i >= (listCDs.size()-1)){
          listCDs.add(new JList());
        }
        listCDs.insertElementAt(new JList(), i);
        if(!(listCDs.get(i) instanceof JList)){
          listCDs.remove(i);
          listCDs.insertElementAt(new JList(), i);
        }
        JList CDtemp = (JList)listCDs.get(i);
        if(i > 0){
          if(listCDs.get(0) instanceof JList){
            JList CDtemp0 = (JList)listCDs.get(0);
            CDtemp.setListData(CDtemp0.getComponents());
            CDtemp.setSelectedIndices(CDtemp0.getSelectedIndices());
          }
        }

        scrollCDlist.getViewport ().setView ((JList)CDtemp);
        scrollCDlist.setPreferredSize (new Dimension (245, 100));

        while(i >= (listMembers.size()-1)){
          listMembers.add(new JList());
        }
        listMembers.insertElementAt(new JList(), i);
        if(!(listMembers.get(i) instanceof JList)){
          listMembers.remove(i);
          listMembers.insertElementAt(new JList(), i);
        }
        final JList membertemp = (JList)listMembers.get(i);
        if(i > 0){
          if(listMembers.get(0) instanceof JList){
            JList membertemp0 = (JList)listMembers.get(0);
            membertemp.setListData(membertemp0.getComponents());
            membertemp.setSelectedIndices(membertemp0.getSelectedIndices());
          }
        }


          scrollCDparticipants.getViewport().setView(membertemp);
          scrollCDparticipants.setPreferredSize(new Dimension(245, 100));

          labelCDlist.setText(" Cooperation Domains");
          labelCDlist.setLabelFor(CDtemp);
          labelCDlist.setPreferredSize(new Dimension(100, 19));

          labelCDparticipants.setText(" CD Participants");
          labelCDparticipants.setLabelFor(membertemp);
          labelCDparticipants.setPreferredSize(new Dimension(65, 19));

          CDtemp.addListSelectionListener(this);
          CDtemp.setToolTipText("Subscribed CDs");
          CDtemp.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
          membertemp.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
          membertemp.setToolTipText("Select participants to receive message");

          /*Synchronize*/
          class synchronizeList
              implements ListSelectionListener {
            JList list;
            Vector objects;
            public synchronizeList(JList list, Vector objects) {
              super();
              this.list = list; //list which is the basis
              this.objects = objects; //lists which should be the same as list
            }

            public void valueChanged(ListSelectionEvent e) {
              Object object;
              for (int j = 0; j < objects.size(); j++) {
                object = objects.get(j);
                if (object instanceof JList) {
                  ListSelectionListener[] listeners = ( (JList) object).
                      getListSelectionListeners();
                  for (int i = 0; i < listeners.length; i++) {
                    ( (JList) object).removeListSelectionListener(listeners[i]);
                  }
                  ( (JList) object).setSelectedIndices( ( (JList) list).
                                                       getSelectedIndices());
                  for (int i = 0; i < listeners.length; i++) {
                    ( (JList) object).addListSelectionListener(listeners[i]);
                  }
                }
              }
            }
          }
          membertemp.addListSelectionListener(new synchronizeList(
              membertemp, listMembers));
          CDtemp.addListSelectionListener(new synchronizeList(
              CDtemp, listCDs));
          
          class SubscriptionMenu implements ActionListener {
        	  JPopupMenu subscribe_menu;
        	  
        	  public SubscriptionMenu() {
        		  subscribe_menu = new JPopupMenu();
    			  JMenuItem subscribe = new JMenuItem("Subscribe to Model");
    			  subscribe.addActionListener(this);
    			  subscribe_menu.add(subscribe);
        	  }
        	  
        	  public void actionPerformed(ActionEvent e) {
        		  URLDescriptor selected = (URLDescriptor)membertemp.getSelectedValue();
        		  if (selected != null) {
        			  view.subscribeToModel(selected);
        		  }
        	  }
        	  
        	  public void show(Component c, int x, int y) {
        		  subscribe_menu.show(c, x, y);
        	  }
        	  
        	  public boolean isVisible() {
        		  return subscribe_menu.isVisible();
        	  }
        	  
        	  public void setVisible(boolean v) {
        		  subscribe_menu.setVisible(v);
        	  }
          }
          
          final SubscriptionMenu sub_menu = new SubscriptionMenu();
          
          membertemp.addMouseListener(new MouseInputAdapter() {
        	  public void mousePressed(MouseEvent e) {
        		  if (e.getButton() == MouseEvent.BUTTON3) {
        			  if (sub_menu.isVisible()) { // this doesn't work (WTF is going on?)
        				  sub_menu.setVisible(false);
        			  }
        			  else {
        				  sub_menu.show(e.getComponent(), e.getX(), e.getY());
        			  }
        		  }
        	  }
          });

        JPanel dummyPanel = new JPanel ();
        dummyPanel.setLayout (new GridLayout (1, 1));
        dummyPanel.add (panelCDtop);

        Vector list = agent.getJoinedCooperationDomains ();
        ((JList)CDtemp).setListData (list);
        if (menuCD_Invite != null) {
          menuCD_Invite.setEnabled (list != null && !list.isEmpty ());
        }

        dummyPanel.setPreferredSize(new Dimension(500, 100));
        return dummyPanel;
      }
}
