package casa.socialcommitments;

import casa.Act;
import casa.agentCom.URLDescriptor;
import casa.conversation2.Conversation;
import casa.event.Event;
import casa.interfaces.PolicyAgentInterface;
import casa.ui.AbstractFadingListModel;
import casa.util.InstanceCounter;
import casa.util.PairComparable;
import casa.util.Trace;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;
import java.util.Set;
import java.util.Vector;
import java.util.concurrent.ConcurrentSkipListMap;
import java.util.concurrent.ConcurrentSkipListSet;

/**
 * <code>SocialCommitmentStore</code> is a ... TODO Add description to JavaDoc file header.
 * @author  Jason Heard
 * @version 0.9
 */
public class SocialCommitmentsStore extends Observable implements Comparable<SocialCommitmentsStore> {
	
	private class SCs extends ConcurrentSkipListSet<SocialCommitment> {
		private static final long serialVersionUID = 9147760729271186069L;
	}
	
	private class Creditor2SCsMap extends ConcurrentSkipListMap<URLDescriptor, SCs> implements Comparable<Creditor2SCsMap>{
		private static final long serialVersionUID = -8534915954242128433L;
		@Override
		public int compareTo(Creditor2SCsMap arg0) {
			for (Iterator<URLDescriptor> me = keySet().iterator(), you = arg0.keySet().iterator(); me.hasNext() && you.hasNext();  ) {
				int ret = me.next().compareTo(you.next());
				if (ret!=0)
					return ret;
			}
			int meSize = keySet().size();
			int youSize = arg0.keySet().size();
			if (meSize<youSize)
				return -1;
			if (meSize>youSize)
				return 1;
			return 0;
		}
	}
	
	/**
	 * Defines a double map:<br>
	 * DeptorURL &rarr; CreditorURL &rarr; [SCs]
	 */
	private class Debtor2Creditor2SCsMap extends ConcurrentSkipListMap<URLDescriptor, Creditor2SCsMap> {
		private static final long serialVersionUID = 8781676121181900124L;
		
		private synchronized Creditor2SCsMap getCreditor2SCsMap (URLDescriptor debtor) {
	    Creditor2SCsMap innerMap = get (debtor);
	    if (innerMap == null) {
	      innerMap = new Creditor2SCsMap ();
	      put (debtor, innerMap);
	    }
	    return innerMap;
	  }

	  private synchronized SCs getSCs (URLDescriptor debtor, URLDescriptor creditor, boolean create) {
	    Creditor2SCsMap innerMap = getCreditor2SCsMap (debtor);
	    SCs commitments = null;
	    if (creditor == null) {
	      commitments = getAllDeptorSCs (debtor);
	    } else {
	      commitments = innerMap.get (creditor);
	    }
	    if (create && commitments == null) {
	      commitments = new SCs ();
	      innerMap.put (creditor, commitments);
	    }
	    return commitments;
	  }

	  private synchronized SCs getAllDeptorSCs (URLDescriptor debtor) {
	    Creditor2SCsMap innerMap = getCreditor2SCsMap (debtor);
	    Set<URLDescriptor> keys = innerMap.keySet ();
	    if (keys.isEmpty())
	      return null;
	    SCs commitments = new SCs ();
	    for (URLDescriptor key: keys) {
	      commitments.addAll (innerMap.get (key));
	    }
	    return commitments;
	  }
	  
	  private synchronized SCs getSCs (SocialCommitment socialCommitment, boolean create) {
	    return getSCs (socialCommitment.getDebtor (), socialCommitment.getCreditor (), create);
	  }

	  private synchronized void addCommitment (SocialCommitment socialCommitment) {
	  	SCs commitments = getSCs (socialCommitment, true);
	    commitments.add (socialCommitment);
	    commitmentsVector.add (socialCommitment);
	  }
	  
	  private synchronized boolean removeCommitment(SocialCommitment sc) {
	  	URLDescriptor debtor = sc.getDebtor();
	  	URLDescriptor creditor = sc.getCreditor();
	  	assert creditor != null : "Creditor must be known to remove a SocialCommitment.";
	  	SCs commitments = getSCs(debtor, creditor, false);
	  	if (commitments==null) 
	  		return false;
	  	return commitments.remove(sc);
	  }
	  
	}
	
  /**
   * Stores all of the commitments. It is a ConcurrentSkipListMap from
   * <code>URLDescriptors</code> (debtor) to <code>ConcurrentSkipListMap</code>. Each inner
   * <code>ConcurrentSkipListMap</code> is from <code>URLDescriptors</code> (creditor) to
   * <code>Vectors</code>. Each <code>Vector</code> contains
   * <code>SocialCommitments</code>.
   */
  private Debtor2Creditor2SCsMap allCommitments = new Debtor2Creditor2SCsMap ();

  /**
   * Redundant storage of all commitments. Used in cases where all commitments
   * need to be recalled.
   */
  private SCs commitmentsVector = new SCs();
  
  private long transactionCount = 0;

  /**
   * Creates a new <code>SocialCommitmentStore</code> object.
   *
   * TODO Fill in specific information for constructor.
   */
  public SocialCommitmentsStore (final PolicyAgentInterface owner) {
  	addObserver(owner);
  	InstanceCounter.add(this);
  }

  public synchronized void addCommitment (SocialCommitment socialCommitment) {
  	allCommitments.addCommitment(socialCommitment);
  	commitmentsVector.add(socialCommitment);
    transactionCount++;
    purgeCheck();
    setChanged();
    notifyObservers(new Operation(Operator.ADD, socialCommitment));
  }
  
  protected void purgeCheck() {
    if (System.currentTimeMillis()>purgeDueTime /*|| Runtime.getRuntime().freeMemory()<10000*/) {
    	int count = allCommitments.size();
    	int purged = purge();
    	Trace.log("info","Purged "+purged+" Social Commitments of "+count);
    }
  }

  /**
   * This creates and adds a commitment with the specified arguments. It uses
   * {@link #addCommitment(SocialCommitment)}.
   * @param agent
   * @param debtor
   * @param creditor
   * @param performative
   * @param act
   * @param event
   * @param action
   * @see {@link SocialCommitment#SocialCommitment(PolicyAgentInterface, URLDescriptor, URLDescriptor, String, Act, Event, Action)}
   * @see #addCommitment(SocialCommitment)
   */
  public void addCommitment (PolicyAgentInterface agent, URLDescriptor debtor, URLDescriptor creditor,
      String performative, Act act, Event event, Action action) {
    SocialCommitment newSC = new SocialCommitment (agent, debtor, creditor, performative, act, event, action);
    addCommitment (newSC);
  }

  /**
   * @param debtor The debtor to filter the returned list. May NOT be null.
   * @param creditor The creditor to filter the returned list. May be null.
   * @return All known {@link SocialCommitment}s that conform to <em>mask</em>.
   */
  public List<SocialCommitment> getCommitments (URLDescriptor debtor, URLDescriptor creditor) {
  	return getCommitments (debtor, creditor, SocialCommitment.MASK_ALL);
  }

  /**
   * @param debtor The debtor to filter the returned list.
   * @param creditor The creditor to filter the returned list.
   * @param mask A mask to filter that state of the {@link SocialCommitment}s returned. 
   * @return All known {@link SocialCommitment}s that conform to <em>mask</em>.
   */
  public List<SocialCommitment> getCommitments (URLDescriptor debtor, URLDescriptor creditor, Set<SocialCommitmentStatusFlags> mask) {
  	SCs tempVector = allCommitments.getSCs (debtor, creditor, false);
  	SCs v = new SCs();
    if (tempVector==null) return new LinkedList<SocialCommitment>(v);
    for (SocialCommitment sc : tempVector) {
      if (sc.meetsMask (mask)) {
        v.add (sc);
      }
    }
    return new LinkedList<SocialCommitment>(v);
  }

  /**
   * Removes <em>socialCommitment</em> from this object.
   * @param socialCommitment
   */
  private synchronized void removeCommitment (SocialCommitment socialCommitment) {
    if (allCommitments.removeCommitment(socialCommitment)) {
      if (!commitmentsVector.remove (socialCommitment))
      	Trace.log("error", "SocialCommitmentStore.removeCommitment(): Cannot remove commitment: "+ socialCommitment);
    }
    else
    	Trace.log("error", "SocialCommitmentStore.removeCommitment(): Cannot remove commitment: "+ socialCommitment);
    transactionCount++;
    purgeCheck();
  }

  /**
   * This removes a commitment formed from the parameters. This uses
   * removeCommitment, to limit the number of functions that actually change
   * things.
   *
   * @param debtor
   * @param creditor
   * @param performative
   * @param act
   * @param msg
   * @param action
   */
  private void removeCommitment (PolicyAgentInterface agent, URLDescriptor debtor, URLDescriptor creditor,
      String performative, Act act, Event event, Action action) {
    SocialCommitment socialCommitment = new SocialCommitment (agent, debtor, creditor,
        performative, act, event, action);
    removeCommitment (socialCommitment);
  }

  /**
	 * @return All known commitments as an unmodifiable collection. 
	 */
  public synchronized Collection<SocialCommitment> getAllCommitments () {
    return Collections.unmodifiableCollection (commitmentsVector);
  }
  
  /**
   * @param id A ConversationID.
   * @return A collection of all {@link SocialCommitment}s associated with <em>id</em> that are still outstanding.
   * @see Conversation#getId()
   * @see SocialCommitment#getOwnerConversationID()
   * @see SocialCommitment#MASK_OUTSTANDING
   */
  public Collection<SocialCommitment> getUnfulfilledCommitments(String id){
  	SCs unfulfilled = new SCs();
  	for (SocialCommitment sc : commitmentsVector)
  		if (sc.meetsMask(SocialCommitment.MASK_OUTSTANDING) && sc.getOwnerConversationID().equals(id))
  			unfulfilled.add(sc);
  	return unfulfilled;
  }

  /**
	 * Retrieves all unfulfilled social commitments, as opposed to just ones
	 * associated with a particular conversation ID
	 * 
	 * @return unfulfilled
	 * @author <a href="mailto:dsbidulo@ucalgary.ca">Daniel Bidulock</a>
	 */
  public Collection<SocialCommitment> getUnfulfilledCommitments(){
  	Collection<SocialCommitment> unfulfilled = new ArrayList<SocialCommitment>();
  	for (SocialCommitment sc : commitmentsVector)
  		if (sc.meetsMask(SocialCommitment.MASK_OUTSTANDING))
  			unfulfilled.add(sc);
  	return unfulfilled;
  }
  
  public enum Operator {ADD, REMOVE};
  public class Operation extends PairComparable<Operator, SocialCommitment> {
  	Operation(Operator op, SocialCommitment sc) {super(op, sc);}
  }

  @Override
  public synchronized String toString () {
    StringBuffer buf = new StringBuffer ();
    for (SocialCommitment sc : getAllCommitments()) {
      buf.append ("  \n").append (sc.toString()).append (';');
    }
    return buf.toString ();
  }

  public synchronized String displayString(Set<SocialCommitmentStatusFlags> mask) {
    StringBuffer buf = new StringBuffer();
    for (SocialCommitment sc : getAllCommitments()) {
      if (sc.meetsMask (mask))
        buf.append("  ").append(sc.toString ()).append(";\n");
    }
    return buf.toString();
  }

  /**
	 * @return
	 */
  public long getTransactionCount () {
    return transactionCount;
  }

  public synchronized int count(Set<SocialCommitmentStatusFlags> mask) {
    int count=0;
    for (SocialCommitment sc : getAllCommitments()) {
    	if (mask==null) 
    	  count++;
    	else {
        if (sc.meetsMask (mask)) count++;
    	}
    }
    return count;
  }

  /**
   * @param mask A mask to filter that state of the {@link SocialCommitment}s. 
   * @return True iff this object contains any {@link SocialCommitment}s that conform to <em>mask</em>.
   */
  public synchronized boolean hasCommitment(Set<SocialCommitmentStatusFlags> mask) {
    // Don't use getAllCommitments for performance reasons
    for (SocialCommitment sc : commitmentsVector) {
      if (sc.meetsMask (mask))
          return true;
    }
    return false;
  }
  
  final static private long purgeInterval = 60000;
  private long purgeDueTime = System.currentTimeMillis()+purgeInterval;
  
  protected synchronized int purge() {
  	purgeDueTime = System.currentTimeMillis()+purgeInterval;
  	int count = 0;
  	final long expiredTime = System.currentTimeMillis() - (AbstractFadingListModel.FADE_TIME+1000);
  	boolean removedOuter = true;
  	Vector<Long> removedSCs = new Vector<Long>();
  	Vector<Long> cantRemove = new Vector<Long>();
  	while (removedOuter) {
  		removedOuter = false;
  		outerLoop:
  			for (Creditor2SCsMap outer: allCommitments.values()) {
  				if (outer.isEmpty()) {
  					allCommitments.values().remove(outer);
						Trace.log("commitments9", "SocialCommitmentStore.purge(): Removed group (from outer) ");
  					removedOuter = true;
  					break outerLoop;
  				}
  				boolean removedInner = true;
  				while (removedInner) {
  					removedInner = false;
  					centreLoop:
  						for (SCs inner: outer.values()) {
  							if (inner.isEmpty()) {
  								outer.values().remove(inner);
  								Trace.log("commitments9", "SocialCommitmentStore.purge(): Removed group (from inner) ");
  								removedInner = true;
  								break centreLoop;
  							}
  							boolean removed = true;
  							while (removed) {
  								removed = false;
  								innerLoop:
  									for (SocialCommitment sc: inner) {
  										long endTime = sc.getEndTime(); 
  										if (endTime!=0 && endTime<expiredTime) {
  											boolean r1 = inner.remove(sc); 
  											if (!r1) {
  												Trace.log("error", "SocialCommitmentStore.purge(): Can't remove (from inner) "+sc);
  												cantRemove.add(sc.getSerialNumber());
  											}
  											else { 
  												Trace.log("commitments8", "SocialCommitmentStore.purge(): Removed (from inner) SN="+sc.getSerialNumber());
  												removedSCs.add(sc.getSerialNumber());
    											removed = true;
  											}
  											boolean r2 = commitmentsVector.remove(sc);
  											if (!r2) {
  												Trace.log("error", "SocialCommitmentStore.purge(): Can't remove (from vector) "+sc);
  											}
  											if (removed) {
  												count++;
  												setChanged();
  												notifyObservers(new Operation(Operator.REMOVE, sc));
  												break innerLoop;
  											}
  										} //if
  										else {
  											Trace.log("commitments9", "SocialCommitmentsStore.purge(): Keeping "+sc);
  										}
  									} //for 
  							} //while
  						} // for
  				} //while
  			} //for
  	} //while
		Trace.log("commitments", "SocialCommitmentStore.purge(): Summary: Removed "+count+" SCs "+removedSCs+", \n\tnow "+commitmentsVector.size()+" SCs left."+(cantRemove.size()==0?"":(" Failed to remove "+cantRemove+".")));
  	return count;
  }

	@Override
	public int compareTo(SocialCommitmentsStore o) {
		return Integer.signum(o.hashCode()-this.hashCode());
	}



}