package casa.util;

import casa.AbstractProcess;
import casa.LAC;
import casa.ui.CustomIcons;

import java.awt.Frame;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;

import javax.net.ssl.SSLEngineResult.Status;
import javax.swing.JOptionPane;

/**
 * Simple debugging tool for the entire CASA package.
 * Set DEBUGGING = true to see all messages sent across the system.
 *
 * <p>Copyright: Copyright (c) 2003, Knowledge Science Group, University of Calgary.
 * Permission to use, copy, modify, distribute and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting documentation.  The
 * Knowledge Science Group makes no representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express or
 * implied warranty.</p>
 *
 * @author <a href="http://cpsc.ucalgary.ca/~kremer/">Rob Kremer</a>
 * @version 0.16
 *
 */

public final class DEBUG {
  private static boolean DEBUGGING = true;

  public static boolean setDEBUGGING() {return setDEBUGGING(true);}
  public static boolean setDEBUGGING(boolean debug) {
    boolean ret = DEBUGGING;
    DEBUGGING = debug;
    return ret;
  }
  public static boolean getDEBUGGING() {return DEBUGGING;}

  public static synchronized void PRINT_ANYWAY( String s) {
  	PRINT_ANYWAY(s, false);
  }
  public static synchronized void PRINT_ANYWAY( String s, boolean error) {
  	AbstractProcess agent = AbstractProcess.getAgent();
  	if (agent!=null && agent.isInitialized()) {
  		agent.println(error?"error":"warning", s);
  	}
  	else {
  		String buf = "[" + Thread.currentThread();
  		for (int i=30-(buf.length()); i>0; i--) buf += ".";
  		buf += CASAUtil.getDateAsString() + "]\n    " + s;
  		System.out.println(buf);
  	}
  }
  public static String getStack(Throwable ex) {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    PrintWriter pw = new PrintWriter(out);
    ex.printStackTrace(pw);
    pw.flush();
    return out.toString();
  }
  public static void PRINT( String s )    { if( DEBUGGING ) PRINT_ANYWAY(s); }
  public static void PRINT( Throwable e ) { PRINT_ANYWAY( "EXCEPTION!>\n"+getStack(e)); }
  public static void PRINT( String s, Throwable e ) { PRINT_ANYWAY( s + ": EXCEPTION!>\n"+getStack(e)); }
  public static void DISPLAY( String s, String title, boolean error )  {
    PRINT_ANYWAY(s, error);
    Frame frame = getFrame();
    try {
      JOptionPane.showMessageDialog (/*frame*/null, s, title, 
          error ? JOptionPane.ERROR_MESSAGE : JOptionPane.INFORMATION_MESSAGE,
              CustomIcons.CASA_SMALL);
    }
    catch (Throwable ex) {
    }
  }
  public static String ASK(String message, String initValue) {
    Frame frame = getFrame();
//    try {
      return JOptionPane.showInputDialog (null, message, initValue); 
//    }
//    catch (Throwable ex) {
//    }
  }
  public static void DISPLAY(Throwable e) { DISPLAY(e,null);}
  public static void DISPLAY(String s) { DISPLAY( s,null); }
  public static void DISPLAY(Throwable e, String title) { DISPLAY( "EXCEPTION!> " + e.toString()+"\n"+getStack(e), title==null?"CASA Exception":title, true); e.printStackTrace();}
  public static void DISPLAY(String s, String title) { DISPLAY( s, title==null?"CASA":title, false); }
  public static void DISPLAY_ERROR(String s) { DISPLAY_ERROR( s, (String)null); }
  public static void DISPLAY_ERROR(String s, Throwable e) { DISPLAY_ERROR( s,null,e);}
  public static void DISPLAY_ERROR(String s, Status e) { DISPLAY_ERROR(s+"\n"+e.toString ()); }
  public static void DISPLAY_ERROR(String s, String title) { DISPLAY( s, title==null?"CASA Error":title, true); }
  public static void DISPLAY_ERROR(String s, String title, Throwable e) { DISPLAY( s+"\n"+getStack(e), "CASA Error", true);}
  private static Frame getFrame() {
    try {
    	if (LAC.ProcessInfo.desktop==null) return null;
			Frame frame = LAC.ProcessInfo.desktop.getJFrame();
			if (frame==null) {
			  Frame[] frames = Frame.getFrames();
			  if (frames.length>0) frame = frames[0];
			}
			return frame;
		} catch (Throwable e) {
			return null;
		}
  }
}
